@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 .. _recipes.rasterio_rgb:
 
@@ -19,15 +18,15 @@ import matplotlib.pyplot as plt
 import xarray as xr
 
 # Read the data
-url = 'https://github.com/mapbox/rasterio/raw/master/tests/data/RGB.byte.tif'
+url = "https://github.com/mapbox/rasterio/raw/master/tests/data/RGB.byte.tif"
 da = xr.open_rasterio(url)
 
 # The data is in UTM projection. We have to set it manually until
 # https://github.com/SciTools/cartopy/issues/813 is implemented
-crs = ccrs.UTM('18N')
+crs = ccrs.UTM("18N")
 
 # Plot on a map
 ax = plt.subplot(projection=crs)
-da.plot.imshow(ax=ax, rgb='band', transform=crs)
-ax.coastlines('10m', color='r')
+da.plot.imshow(ax=ax, rgb="band", transform=crs)
+ax.coastlines("10m", color="r")
 plt.show()
