@@ -1,5 +1,4 @@
 import copy
-from collections import OrderedDict
 
 import numpy as np
 
@@ -16,8 +15,8 @@ class InMemoryDataStore(AbstractWritableDataStore):
     """
 
     def __init__(self, variables=None, attributes=None):
-        self._variables = OrderedDict() if variables is None else variables
-        self._attributes = OrderedDict() if attributes is None else attributes
+        self._variables = {} if variables is None else variables
+        self._attributes = {} if attributes is None else attributes
 
     def get_attrs(self):
         return self._attributes
@@ -26,7 +25,7 @@ class InMemoryDataStore(AbstractWritableDataStore):
         return self._variables
 
     def get_dimensions(self):
-        dims = OrderedDict()
+        dims = {}
         for v in self._variables.values():
             for d, s in v.dims.items():
                 dims[d] = s
