@@ -1,10 +1,8 @@
-from collections import OrderedDict
-
 import numpy as np
 
-from .. import Variable
 from ..core import indexing
-from ..core.utils import Frozen, FrozenOrderedDict
+from ..core.utils import Frozen, FrozenDict
+from ..core.variable import Variable
 from .common import AbstractDataStore, BackendArray
 from .file_manager import CachingFileManager
 from .locks import HDF5_LOCK, NETCDFC_LOCK, combine_locks, ensure_lock
@@ -14,7 +12,6 @@ PNETCDF_LOCK = combine_locks([HDF5_LOCK, NETCDFC_LOCK])
 
 
 class PncArrayWrapper(BackendArray):
-
     def __init__(self, variable_name, datastore):
         self.datastore = datastore
         self.variable_name = variable_name
@@ -28,8 +25,8 @@ class PncArrayWrapper(BackendArray):
 
     def __getitem__(self, key):
         return indexing.explicit_indexing_adapter(
-            key, self.shape, indexing.IndexingSupport.OUTER_1VECTOR,
-            self._getitem)
+            key, self.shape, indexing.IndexingSupport.OUTER_1VECTOR, self._getitem
+        )
 
     def _getitem(self, key):
         with self.datastore.lock:
@@ -40,15 +37,15 @@ class PncArrayWrapper(BackendArray):
 class PseudoNetCDFDataStore(AbstractDataStore):
     """Store for accessing datasets via PseudoNetCDF
     """
+
     @classmethod
-    def open(cls, filename, lock=None, **format_kwds):
+    def open(cls, filename, lock=None, mode=None, **format_kwargs):
         from PseudoNetCDF import pncopen
 
-        keywords = dict(kwargs=format_kwds)
+        keywords = {"kwargs": format_kwargs}
         # only include mode if explicitly passed
-        mode = format_kwds.pop('mode', None)
         if mode is not None:
-            keywords['mode'] = mode
+            keywords["mode"] = mode
 
         if lock is None:
             lock = PNETCDF_LOCK
@@ -65,29 +62,27 @@ class PseudoNetCDFDataStore(AbstractDataStore):
         return self._manager.acquire()
 
     def open_store_variable(self, name, var):
-        data = indexing.LazilyOuterIndexedArray(
-            PncArrayWrapper(name, self)
-        )
-        attrs = OrderedDict((k, getattr(var, k)) for k in var.ncattrs())
+        data = indexing.LazilyOuterIndexedArray(PncArrayWrapper(name, self))
+        attrs = {k: getattr(var, k) for k in var.ncattrs()}
         return Variable(var.dimensions, data, attrs)
 
     def get_variables(self):
-        return FrozenOrderedDict((k, self.open_store_variable(k, v))
-                                 for k, v in self.ds.variables.items())
+        return FrozenDict(
+            (k, self.open_store_variable(k, v)) for k, v in self.ds.variables.items()
+        )
 
     def get_attrs(self):
-        return Frozen(dict([(k, getattr(self.ds, k))
-                            for k in self.ds.ncattrs()]))
+        return Frozen({k: getattr(self.ds, k) for k in self.ds.ncattrs()})
 
     def get_dimensions(self):
         return Frozen(self.ds.dimensions)
 
     def get_encoding(self):
-        encoding = {}
-        encoding['unlimited_dims'] = set(
-            [k for k in self.ds.dimensions
-             if self.ds.dimensions[k].isunlimited()])
-        return encoding
+        return {
+            "unlimited_dims": {
+                k for k in self.ds.dimensions if self.ds.dimensions[k].isunlimited()
+            }
+        }
 
     def close(self):
         self._manager.close()
