@@ -5,7 +5,7 @@ import numpy as np
 from . import utils
 
 # Use as a sentinel value to indicate a dtype appropriate NA value.
-NA = utils.ReprObject('<NA>')
+NA = utils.ReprObject("<NA>")
 
 
 @functools.total_ordering
@@ -61,7 +61,7 @@ def maybe_promote(dtype):
         # See https://github.com/numpy/numpy/issues/10685
         # np.timedelta64 is a subclass of np.integer
         # Check np.timedelta64 before np.integer
-        fill_value = np.timedelta64('NaT')
+        fill_value = np.timedelta64("NaT")
     elif np.issubdtype(dtype, np.integer):
         if dtype.itemsize <= 2:
             dtype = np.float32
@@ -71,14 +71,14 @@ def maybe_promote(dtype):
     elif np.issubdtype(dtype, np.complexfloating):
         fill_value = np.nan + np.nan * 1j
     elif np.issubdtype(dtype, np.datetime64):
-        fill_value = np.datetime64('NaT')
+        fill_value = np.datetime64("NaT")
     else:
         dtype = object
         fill_value = np.nan
     return np.dtype(dtype), fill_value
 
 
-NAT_TYPES = (np.datetime64('NaT'), np.timedelta64('NaT'))
+NAT_TYPES = (np.datetime64("NaT"), np.timedelta64("NaT"))
 
 
 def get_fill_value(dtype):
@@ -139,8 +139,7 @@ def get_neg_infinity(dtype):
 def is_datetime_like(dtype):
     """Check if a dtype is a subclass of the numpy datetime types
     """
-    return (np.issubdtype(dtype, np.datetime64) or
-            np.issubdtype(dtype, np.timedelta64))
+    return np.issubdtype(dtype, np.datetime64) or np.issubdtype(dtype, np.timedelta64)
 
 
 def result_type(*arrays_and_dtypes):
@@ -162,8 +161,9 @@ def result_type(*arrays_and_dtypes):
     types = {np.result_type(t).type for t in arrays_and_dtypes}
 
     for left, right in PROMOTE_TO_OBJECT:
-        if (any(issubclass(t, left) for t in types) and
-                any(issubclass(t, right) for t in types)):
+        if any(issubclass(t, left) for t in types) and any(
+            issubclass(t, right) for t in types
+        ):
             return np.dtype(object)
 
     return np.result_type(*arrays_and_dtypes)
