@@ -5,16 +5,37 @@ import warnings
 import numpy as np
 import pandas as pd
 
-from . import dtypes, duck_array_ops, nputils, ops, utils
+from . import dtypes, duck_array_ops, nputils, ops
 from .arithmetic import SupportsArithmetic
+from .common import ImplementsArrayReduce, ImplementsDatasetReduce
 from .concat import concat
-from .common import ALL_DIMS, ImplementsArrayReduce, ImplementsDatasetReduce
+from .formatting import format_array_flat
+from .indexes import propagate_indexes
 from .options import _get_keep_attrs
 from .pycompat import integer_types
-from .utils import hashable, maybe_wrap_array, peek_at, safe_cast_to_index
+from .utils import (
+    either_dict_or_kwargs,
+    hashable,
+    is_scalar,
+    maybe_wrap_array,
+    peek_at,
+    safe_cast_to_index,
+)
 from .variable import IndexVariable, Variable, as_variable
 
 
+def check_reduce_dims(reduce_dims, dimensions):
+
+    if reduce_dims is not ...:
+        if is_scalar(reduce_dims):
+            reduce_dims = [reduce_dims]
+        if any([dim not in dimensions for dim in reduce_dims]):
+            raise ValueError(
+                "cannot reduce over dimensions %r. expected either '...' to reduce over all dimensions or one or more of %r."
+                % (reduce_dims, dimensions)
+            )
+
+
 def unique_value_groups(ar, sort=True):
     """Group an array by its unique values.
 
@@ -45,21 +66,32 @@ def unique_value_groups(ar, sort=True):
 def _dummy_copy(xarray_obj):
     from .dataset import Dataset
     from .dataarray import DataArray
+
     if isinstance(xarray_obj, Dataset):
-        res = Dataset(dict((k, dtypes.get_fill_value(v.dtype))
-                           for k, v in xarray_obj.data_vars.items()),
-                      dict((k, dtypes.get_fill_value(v.dtype))
-                           for k, v in xarray_obj.coords.items()
-                           if k not in xarray_obj.dims),
-                      xarray_obj.attrs)
+        res = Dataset(
+            {
+                k: dtypes.get_fill_value(v.dtype)
+                for k, v in xarray_obj.data_vars.items()
+            },
+            {
+                k: dtypes.get_fill_value(v.dtype)
+                for k, v in xarray_obj.coords.items()
+                if k not in xarray_obj.dims
+            },
+            xarray_obj.attrs,
+        )
     elif isinstance(xarray_obj, DataArray):
-        res = DataArray(dtypes.get_fill_value(xarray_obj.dtype),
-                        dict((k, dtypes.get_fill_value(v.dtype))
-                             for k, v in xarray_obj.coords.items()
-                             if k not in xarray_obj.dims),
-                        dims=[],
-                        name=xarray_obj.name,
-                        attrs=xarray_obj.attrs)
+        res = DataArray(
+            dtypes.get_fill_value(xarray_obj.dtype),
+            {
+                k: dtypes.get_fill_value(v.dtype)
+                for k, v in xarray_obj.coords.items()
+                if k not in xarray_obj.dims
+            },
+            dims=[],
+            name=xarray_obj.name,
+            attrs=xarray_obj.attrs,
+        )
     else:  # pragma: no cover
         raise AssertionError
     return res
@@ -76,10 +108,13 @@ def _consolidate_slices(slices):
     last_slice = slice(None)
     for slice_ in slices:
         if not isinstance(slice_, slice):
-            raise ValueError('list element is not a slice: %r' % slice_)
-        if (result and last_slice.stop == slice_.start and
-                _is_one_or_none(last_slice.step) and
-                _is_one_or_none(slice_.step)):
+            raise ValueError("list element is not a slice: %r" % slice_)
+        if (
+            result
+            and last_slice.stop == slice_.start
+            and _is_one_or_none(last_slice.step)
+            and _is_one_or_none(slice_.step)
+        ):
             last_slice = slice(last_slice.start, slice_.stop, slice_.step)
             result[-1] = last_slice
         else:
@@ -119,20 +154,40 @@ class _DummyGroup:
     Should not be user visible.
     """
 
+    __slots__ = ("name", "coords", "size")
+
     def __init__(self, obj, name, coords):
         self.name = name
         self.coords = coords
-        self.dims = (name,)
-        self.ndim = 1
         self.size = obj.sizes[name]
-        self.values = range(self.size)
+
+    @property
+    def dims(self):
+        return (self.name,)
+
+    @property
+    def ndim(self):
+        return 1
+
+    @property
+    def values(self):
+        return range(self.size)
+
+    @property
+    def shape(self):
+        return (self.size,)
+
+    def __getitem__(self, key):
+        if isinstance(key, tuple):
+            key = key[0]
+        return self.values[key]
 
 
 def _ensure_1d(group, obj):
     if group.ndim != 1:
         # try to stack the dims of the group into a single dim
         orig_dims = group.dims
-        stacked_dim = 'stacked_' + '_'.join(orig_dims)
+        stacked_dim = "stacked_" + "_".join(orig_dims)
         # these dimensions get created by the stack operation
         inserted_dims = [dim for dim in group.dims if dim not in group.coords]
         # the copy is necessary here, otherwise read only array raises error
@@ -196,8 +251,30 @@ class GroupBy(SupportsArithmetic):
     DataArray.groupby
     """
 
-    def __init__(self, obj, group, squeeze=False, grouper=None, bins=None,
-                 restore_coord_dims=None, cut_kwargs={}):
+    __slots__ = (
+        "_full_index",
+        "_inserted_dims",
+        "_group",
+        "_group_dim",
+        "_group_indices",
+        "_groups",
+        "_obj",
+        "_restore_coord_dims",
+        "_stacked_dim",
+        "_unique_coord",
+        "_dims",
+    )
+
+    def __init__(
+        self,
+        obj,
+        group,
+        squeeze=False,
+        grouper=None,
+        bins=None,
+        restore_coord_dims=None,
+        cut_kwargs={},
+    ):
         """Create a GroupBy object
 
         Parameters
@@ -229,30 +306,39 @@ class GroupBy(SupportsArithmetic):
 
         if not isinstance(group, (DataArray, IndexVariable)):
             if not hashable(group):
-                raise TypeError('`group` must be an xarray.DataArray or the '
-                                'name of an xarray variable or dimension')
+                raise TypeError(
+                    "`group` must be an xarray.DataArray or the "
+                    "name of an xarray variable or dimension"
+                )
             group = obj[group]
+            if len(group) == 0:
+                raise ValueError(f"{group.name} must not be empty")
+
             if group.name not in obj.coords and group.name in obj.dims:
                 # DummyGroups should not appear on groupby results
                 group = _DummyGroup(obj, group.name, group.coords)
 
-        if getattr(group, 'name', None) is None:
-            raise ValueError('`group` must have a name')
+        if getattr(group, "name", None) is None:
+            raise ValueError("`group` must have a name")
 
         group, obj, stacked_dim, inserted_dims = _ensure_1d(group, obj)
-        group_dim, = group.dims
+        (group_dim,) = group.dims
 
         expected_size = obj.sizes[group_dim]
         if group.size != expected_size:
-            raise ValueError('the group variable\'s length does not '
-                             'match the length of this variable along its '
-                             'dimension')
+            raise ValueError(
+                "the group variable's length does not "
+                "match the length of this variable along its "
+                "dimension"
+            )
 
         full_index = None
 
         if bins is not None:
+            if duck_array_ops.isnull(bins).all():
+                raise ValueError("All bin edges are NaN.")
             binned = pd.cut(group.values, bins, **cut_kwargs)
-            new_dim_name = group.name + '_bins'
+            new_dim_name = group.name + "_bins"
             group = DataArray(binned, group.coords, name=new_dim_name)
             full_index = binned.categories
 
@@ -260,13 +346,12 @@ class GroupBy(SupportsArithmetic):
             index = safe_cast_to_index(group)
             if not index.is_monotonic:
                 # TODO: sort instead of raising an error
-                raise ValueError('index must be monotonic for resampling')
-            full_index, first_items = self._get_index_and_items(
-                index, grouper)
+                raise ValueError("index must be monotonic for resampling")
+            full_index, first_items = self._get_index_and_items(index, grouper)
             sbins = first_items.values.astype(np.int64)
-            group_indices = ([slice(i, j)
-                              for i, j in zip(sbins[:-1], sbins[1:])] +
-                             [slice(sbins[-1], None)])
+            group_indices = [slice(i, j) for i, j in zip(sbins[:-1], sbins[1:])] + [
+                slice(sbins[-1], None)
+            ]
             unique_coord = IndexVariable(group.name, first_items.index)
         elif group.dims == (group.name,) and _unique_and_monotonic(group):
             # no need to factorize
@@ -277,19 +362,42 @@ class GroupBy(SupportsArithmetic):
                 group_indices = [slice(i, i + 1) for i in group_indices]
             unique_coord = group
         else:
+            if group.isnull().any():
+                # drop any NaN valued groups.
+                # also drop obj values where group was NaN
+                # Use where instead of reindex to account for duplicate coordinate labels.
+                obj = obj.where(group.notnull(), drop=True)
+                group = group.dropna(group_dim)
+
             # look through group to find the unique values
             unique_values, group_indices = unique_value_groups(
-                safe_cast_to_index(group), sort=(bins is None))
+                safe_cast_to_index(group), sort=(bins is None)
+            )
             unique_coord = IndexVariable(group.name, unique_values)
 
-        if isinstance(obj, DataArray) \
-                and restore_coord_dims is None \
-                and any(obj[c].ndim > 1 for c in obj.coords):
-            warnings.warn('This DataArray contains multi-dimensional '
-                          'coordinates. In the future, the dimension order '
-                          'of these coordinates will be restored as well '
-                          'unless you specify restore_coord_dims=False.',
-                          FutureWarning, stacklevel=2)
+        if len(group_indices) == 0:
+            if bins is not None:
+                raise ValueError(
+                    "None of the data falls within bins with edges %r" % bins
+                )
+            else:
+                raise ValueError(
+                    "Failed to group data. Are you grouping by a variable that is all NaN?"
+                )
+
+        if (
+            isinstance(obj, DataArray)
+            and restore_coord_dims is None
+            and any(obj[c].ndim > 1 for c in obj.coords)
+        ):
+            warnings.warn(
+                "This DataArray contains multi-dimensional "
+                "coordinates. In the future, the dimension order "
+                "of these coordinates will be restored as well "
+                "unless you specify restore_coord_dims=False.",
+                FutureWarning,
+                stacklevel=2,
+            )
             restore_coord_dims = False
 
         # specification for the groupby operation
@@ -305,13 +413,22 @@ class GroupBy(SupportsArithmetic):
 
         # cached attributes
         self._groups = None
+        self._dims = None
+
+    @property
+    def dims(self):
+        if self._dims is None:
+            self._dims = self._obj.isel(
+                **{self._group_dim: self._group_indices[0]}
+            ).dims
+
+        return self._dims
 
     @property
     def groups(self):
         # provided to mimic pandas.groupby
         if self._groups is None:
-            self._groups = dict(zip(self._unique_coord.values,
-                                    self._group_indices))
+            self._groups = dict(zip(self._unique_coord.values, self._group_indices))
         return self._groups
 
     def __len__(self):
@@ -320,8 +437,17 @@ class GroupBy(SupportsArithmetic):
     def __iter__(self):
         return zip(self._unique_coord.values, self._iter_grouped())
 
+    def __repr__(self):
+        return "{}, grouped over {!r} \n{!r} groups with labels {}.".format(
+            self.__class__.__name__,
+            self._unique_coord.name,
+            self._unique_coord.size,
+            ", ".join(format_array_flat(self._unique_coord, 30).split()),
+        )
+
     def _get_index_and_items(self, index, grouper):
         from .resample_cftime import CFTimeGrouper
+
         s = pd.Series(np.arange(index.size), index)
         if isinstance(grouper, CFTimeGrouper):
             first_items = grouper.first_items(index)
@@ -345,7 +471,7 @@ class GroupBy(SupportsArithmetic):
         else:
             coord = self._unique_coord
             positions = None
-        dim, = coord.dims
+        (dim,) = coord.dims
         if isinstance(coord, _DummyGroup):
             coord = None
         return coord, dim, positions
@@ -358,6 +484,7 @@ class GroupBy(SupportsArithmetic):
             applied = self._yield_binary_applied(g, other)
             combined = self._combine(applied)
             return combined
+
         return func
 
     def _yield_binary_applied(self, func, other):
@@ -367,15 +494,18 @@ class GroupBy(SupportsArithmetic):
             try:
                 other_sel = other.sel(**{self._group.name: group_value})
             except AttributeError:
-                raise TypeError('GroupBy objects only support binary ops '
-                                'when the other argument is a Dataset or '
-                                'DataArray')
+                raise TypeError(
+                    "GroupBy objects only support binary ops "
+                    "when the other argument is a Dataset or "
+                    "DataArray"
+                )
             except (KeyError, ValueError):
                 if self._group.name not in other.dims:
-                    raise ValueError('incompatible dimensions for a grouped '
-                                     'binary operation: the group variable %r '
-                                     'is not a dimension on the other argument'
-                                     % self._group.name)
+                    raise ValueError(
+                        "incompatible dimensions for a grouped "
+                        "binary operation: the group variable %r "
+                        "is not a dimension on the other argument" % self._group.name
+                    )
                 if dummy is None:
                     dummy = _dummy_copy(other)
                 other_sel = dummy
@@ -387,8 +517,7 @@ class GroupBy(SupportsArithmetic):
         """Our index contained empty groups (e.g., from a resampling). If we
         reduced on that dimension, we want to restore the full index.
         """
-        if (self._full_index is not None and
-                self._group.name in combined.dims):
+        if self._full_index is not None and self._group.name in combined.dims:
             indexers = {self._group.name: self._full_index}
             combined = combined.reindex(**indexers)
         return combined
@@ -401,6 +530,7 @@ class GroupBy(SupportsArithmetic):
             for dim in self._inserted_dims:
                 if dim in obj.coords:
                     del obj.coords[dim]
+            obj._indexes = propagate_indexes(obj._indexes, exclude=self._inserted_dims)
         return obj
 
     def fillna(self, value):
@@ -428,6 +558,108 @@ class GroupBy(SupportsArithmetic):
         out = ops.fillna(self, value)
         return out
 
+    def quantile(self, q, dim=None, interpolation="linear", keep_attrs=None):
+        """Compute the qth quantile over each array in the groups and
+        concatenate them together into a new array.
+
+        Parameters
+        ----------
+        q : float in range of [0,1] (or sequence of floats)
+            Quantile to compute, which must be between 0 and 1
+            inclusive.
+        dim : `...`, str or sequence of str, optional
+            Dimension(s) over which to apply quantile.
+            Defaults to the grouped dimension.
+        interpolation : {'linear', 'lower', 'higher', 'midpoint', 'nearest'}
+            This optional parameter specifies the interpolation method to
+            use when the desired quantile lies between two data points
+            ``i < j``:
+
+                * linear: ``i + (j - i) * fraction``, where ``fraction`` is
+                  the fractional part of the index surrounded by ``i`` and
+                  ``j``.
+                * lower: ``i``.
+                * higher: ``j``.
+                * nearest: ``i`` or ``j``, whichever is nearest.
+                * midpoint: ``(i + j) / 2``.
+
+        Returns
+        -------
+        quantiles : Variable
+            If `q` is a single quantile, then the result is a
+            scalar. If multiple percentiles are given, first axis of
+            the result corresponds to the quantile. In either case a
+            quantile dimension is added to the return array. The other
+            dimensions are the dimensions that remain after the
+            reduction of the array.
+
+        See Also
+        --------
+        numpy.nanquantile, pandas.Series.quantile, Dataset.quantile,
+        DataArray.quantile
+
+        Examples
+        --------
+
+        >>> da = xr.DataArray(
+        ...     [[1.3, 8.4, 0.7, 6.9], [0.7, 4.2, 9.4, 1.5], [6.5, 7.3, 2.6, 1.9]],
+        ...     coords={"x": [0, 0, 1], "y": [1, 1, 2, 2]},
+        ...     dims=("y", "y"),
+        ... )
+        >>> ds = xr.Dataset({"a": da})
+        >>> da.groupby("x").quantile(0)
+        <xarray.DataArray (x: 2, y: 4)>
+        array([[0.7, 4.2, 0.7, 1.5],
+               [6.5, 7.3, 2.6, 1.9]])
+        Coordinates:
+            quantile  float64 0.0
+          * y         (y) int64 1 1 2 2
+          * x         (x) int64 0 1
+        >>> ds.groupby("y").quantile(0, dim=...)
+        <xarray.Dataset>
+        Dimensions:   (y: 2)
+        Coordinates:
+            quantile  float64 0.0
+          * y         (y) int64 1 2
+        Data variables:
+            a         (y) float64 0.7 0.7
+        >>> da.groupby("x").quantile([0, 0.5, 1])
+        <xarray.DataArray (x: 2, y: 4, quantile: 3)>
+        array([[[0.7 , 1.  , 1.3 ],
+                [4.2 , 6.3 , 8.4 ],
+                [0.7 , 5.05, 9.4 ],
+                [1.5 , 4.2 , 6.9 ]],
+               [[6.5 , 6.5 , 6.5 ],
+                [7.3 , 7.3 , 7.3 ],
+                [2.6 , 2.6 , 2.6 ],
+                [1.9 , 1.9 , 1.9 ]]])
+        Coordinates:
+          * y         (y) int64 1 1 2 2
+          * quantile  (quantile) float64 0.0 0.5 1.0
+          * x         (x) int64 0 1
+        >>> ds.groupby("y").quantile([0, 0.5, 1], dim=...)
+        <xarray.Dataset>
+        Dimensions:   (quantile: 3, y: 2)
+        Coordinates:
+          * quantile  (quantile) float64 0.0 0.5 1.0
+          * y         (y) int64 1 2
+        Data variables:
+            a         (y, quantile) float64 0.7 5.35 8.4 0.7 2.25 9.4
+        """
+        if dim is None:
+            dim = self._group_dim
+
+        out = self.map(
+            self._obj.__class__.quantile,
+            shortcut=False,
+            q=q,
+            dim=dim,
+            interpolation=interpolation,
+            keep_attrs=keep_attrs,
+        )
+
+        return out
+
     def where(self, cond, other=dtypes.NA):
         """Return elements from `self` or `other` depending on `cond`.
 
@@ -456,8 +688,7 @@ class GroupBy(SupportsArithmetic):
             return self._obj
         if keep_attrs is None:
             keep_attrs = _get_keep_attrs(default=True)
-        return self.reduce(op, self._group_dim, skipna=skipna,
-                           keep_attrs=keep_attrs, allow_lazy=True)
+        return self.reduce(op, self._group_dim, skipna=skipna, keep_attrs=keep_attrs)
 
     def first(self, skipna=None, keep_attrs=None):
         """Return the first element of each group along the group dimension
@@ -469,7 +700,7 @@ class GroupBy(SupportsArithmetic):
         """
         return self._first_or_last(duck_array_ops.last, skipna, keep_attrs)
 
-    def assign_coords(self, **kwargs):
+    def assign_coords(self, coords=None, **coords_kwargs):
         """Assign coordinates by group.
 
         See also
@@ -477,7 +708,8 @@ class GroupBy(SupportsArithmetic):
         Dataset.assign_coords
         Dataset.swap_dims
         """
-        return self.apply(lambda ds: ds.assign_coords(**kwargs))
+        coords_kwargs = either_dict_or_kwargs(coords, coords_kwargs, "assign_coords")
+        return self.map(lambda ds: ds.assign_coords(**coords_kwargs))
 
 
 def _maybe_reorder(xarray_obj, dim, positions):
@@ -514,7 +746,7 @@ class DataArrayGroupBy(GroupBy, ImplementsArrayReduce):
     def _restore_dim_order(self, stacked):
         def lookup_order(dimension):
             if dimension == self._group.name:
-                dimension, = self._group.dims
+                (dimension,) = self._group.dims
             if dimension in self._obj.dims:
                 axis = self._obj.get_axis_num(dimension)
             else:
@@ -522,11 +754,10 @@ class DataArrayGroupBy(GroupBy, ImplementsArrayReduce):
             return axis
 
         new_order = sorted(stacked.dims, key=lookup_order)
-        return stacked.transpose(
-            *new_order, transpose_coords=self._restore_coord_dims)
+        return stacked.transpose(*new_order, transpose_coords=self._restore_coord_dims)
 
-    def apply(self, func, shortcut=False, args=(), **kwargs):
-        """Apply a function over each array in the group and concatenate them
+    def map(self, func, shortcut=False, args=(), **kwargs):
+        """Apply a function to each array in the group and concatenate them
         together into a new array.
 
         `func` is called like `func(ar, *args, **kwargs)` for each array `ar`
@@ -547,17 +778,19 @@ class DataArrayGroupBy(GroupBy, ImplementsArrayReduce):
             Callable to apply to each array.
         shortcut : bool, optional
             Whether or not to shortcut evaluation under the assumptions that:
+
             (1) The action of `func` does not depend on any of the array
                 metadata (attributes or coordinates) but only on the data and
                 dimensions.
             (2) The action of `func` creates arrays with homogeneous metadata,
                 that is, with the same dimensions and attributes.
+
             If these conditions are satisfied `shortcut` provides significant
             speedup. This should be the case for many common groupby operations
             (e.g., applying numpy ufuncs).
-        args : tuple, optional
+        ``*args`` : tuple, optional
             Positional arguments passed to `func`.
-        **kwargs
+        ``**kwargs``
             Used to call `func(ar, **kwargs)` for each array `ar`.
 
         Returns
@@ -569,10 +802,24 @@ class DataArrayGroupBy(GroupBy, ImplementsArrayReduce):
             grouped = self._iter_grouped_shortcut()
         else:
             grouped = self._iter_grouped()
-        applied = (maybe_wrap_array(arr, func(arr, *args, **kwargs))
-                   for arr in grouped)
+        applied = (maybe_wrap_array(arr, func(arr, *args, **kwargs)) for arr in grouped)
         return self._combine(applied, shortcut=shortcut)
 
+    def apply(self, func, shortcut=False, args=(), **kwargs):
+        """
+        Backward compatible implementation of ``map``
+
+        See Also
+        --------
+        DataArrayGroupBy.map
+        """
+        warnings.warn(
+            "GroupBy.apply may be deprecated in the future. Using GroupBy.map is encouraged",
+            PendingDeprecationWarning,
+            stacklevel=2,
+        )
+        return self.map(func, shortcut=shortcut, args=args, **kwargs)
+
     def _combine(self, applied, restore_coord_dims=False, shortcut=False):
         """Recombine the applied objects like the original."""
         applied_example, applied = peek_at(applied)
@@ -588,73 +835,17 @@ class DataArrayGroupBy(GroupBy, ImplementsArrayReduce):
             combined = self._restore_dim_order(combined)
         if coord is not None:
             if shortcut:
-                combined._coords[coord.name] = as_variable(coord)
+                coord_var = as_variable(coord)
+                combined._coords[coord.name] = coord_var
             else:
                 combined.coords[coord.name] = coord
         combined = self._maybe_restore_empty_groups(combined)
         combined = self._maybe_unstack(combined)
         return combined
 
-    def quantile(self, q, dim=None, interpolation='linear', keep_attrs=None):
-        """Compute the qth quantile over each array in the groups and
-        concatenate them together into a new array.
-
-        Parameters
-        ----------
-        q : float in range of [0,1] (or sequence of floats)
-            Quantile to compute, which must be between 0 and 1
-            inclusive.
-        dim : str or sequence of str, optional
-            Dimension(s) over which to apply quantile.
-            Defaults to the grouped dimension.
-        interpolation : {'linear', 'lower', 'higher', 'midpoint', 'nearest'}
-            This optional parameter specifies the interpolation method to
-            use when the desired quantile lies between two data points
-            ``i < j``:
-                * linear: ``i + (j - i) * fraction``, where ``fraction`` is
-                  the fractional part of the index surrounded by ``i`` and
-                  ``j``.
-                * lower: ``i``.
-                * higher: ``j``.
-                * nearest: ``i`` or ``j``, whichever is nearest.
-                * midpoint: ``(i + j) / 2``.
-
-        Returns
-        -------
-        quantiles : Variable
-            If `q` is a single quantile, then the result
-            is a scalar. If multiple percentiles are given, first axis of
-            the result corresponds to the quantile and a quantile dimension
-            is added to the return array. The other dimensions are the
-            dimensions that remain after the reduction of the array.
-
-        See Also
-        --------
-        numpy.nanpercentile, pandas.Series.quantile, Dataset.quantile,
-        DataArray.quantile
-        """
-        if dim == DEFAULT_DIMS:
-            dim = ALL_DIMS
-            # TODO change this to dim = self._group_dim after
-            # the deprecation process
-            if self._obj.ndim > 1:
-                warnings.warn(
-                    "Default reduction dimension will be changed to the "
-                    "grouped dimension in a future version of xarray. To "
-                    "silence this warning, pass dim=xarray.ALL_DIMS "
-                    "explicitly.",
-                    FutureWarning, stacklevel=2)
-
-        out = self.apply(self._obj.__class__.quantile, shortcut=False,
-                         q=q, dim=dim, interpolation=interpolation,
-                         keep_attrs=keep_attrs)
-
-        if np.asarray(q, dtype=np.float64).ndim == 0:
-            out = out.drop('quantile')
-        return out
-
-    def reduce(self, func, dim=None, axis=None, keep_attrs=None,
-               shortcut=True, **kwargs):
+    def reduce(
+        self, func, dim=None, axis=None, keep_attrs=None, shortcut=True, **kwargs
+    ):
         """Reduce the items in this group by applying `func` along some
         dimension(s).
 
@@ -664,7 +855,7 @@ class DataArrayGroupBy(GroupBy, ImplementsArrayReduce):
             Function which can be called in the form
             `func(x, axis=axis, **kwargs)` to return the result of collapsing
             an np.ndarray over an integer valued axis.
-        dim : str or sequence of str, optional
+        dim : `...`, str or sequence of str, optional
             Dimension(s) over which to apply `func`.
         axis : int or sequence of int, optional
             Axis(es) over which to apply `func`. Only one of the 'dimension'
@@ -683,51 +874,27 @@ class DataArrayGroupBy(GroupBy, ImplementsArrayReduce):
             Array with summarized data and the indicated dimension(s)
             removed.
         """
-        if dim == DEFAULT_DIMS:
-            dim = ALL_DIMS
-            # TODO change this to dim = self._group_dim after
-            # the deprecation process
-            if self._obj.ndim > 1:
-                warnings.warn(
-                    "Default reduction dimension will be changed to the "
-                    "grouped dimension in a future version of xarray. To "
-                    "silence this warning, pass dim=xarray.ALL_DIMS "
-                    "explicitly.",
-                    FutureWarning, stacklevel=2)
+        if dim is None:
+            dim = self._group_dim
 
         if keep_attrs is None:
             keep_attrs = _get_keep_attrs(default=False)
 
         def reduce_array(ar):
             return ar.reduce(func, dim, axis, keep_attrs=keep_attrs, **kwargs)
-        return self.apply(reduce_array, shortcut=shortcut)
 
-    # TODO remove the following class method and DEFAULT_DIMS after the
-    # deprecation cycle
-    @classmethod
-    def _reduce_method(cls, func, include_skipna, numeric_only):
-        if include_skipna:
-            def wrapped_func(self, dim=DEFAULT_DIMS, axis=None, skipna=None,
-                             keep_attrs=None, **kwargs):
-                return self.reduce(func, dim, axis, keep_attrs=keep_attrs,
-                                   skipna=skipna, allow_lazy=True, **kwargs)
-        else:
-            def wrapped_func(self, dim=DEFAULT_DIMS, axis=None,  # type: ignore
-                             keep_attrs=None, **kwargs):
-                return self.reduce(func, dim, axis, keep_attrs=keep_attrs,
-                                   allow_lazy=True, **kwargs)
-        return wrapped_func
+        check_reduce_dims(dim, self.dims)
 
+        return self.map(reduce_array, shortcut=shortcut)
 
-DEFAULT_DIMS = utils.ReprObject('<default-dims>')
 
 ops.inject_reduce_methods(DataArrayGroupBy)
 ops.inject_binary_ops(DataArrayGroupBy)
 
 
 class DatasetGroupBy(GroupBy, ImplementsDatasetReduce):
-    def apply(self, func, args=(), shortcut=None, **kwargs):
-        """Apply a function over each Dataset in the group and concatenate them
+    def map(self, func, args=(), shortcut=None, **kwargs):
+        """Apply a function to each Dataset in the group and concatenate them
         together into a new Dataset.
 
         `func` is called like `func(ds, *args, **kwargs)` for each dataset `ds`
@@ -760,6 +927,22 @@ class DatasetGroupBy(GroupBy, ImplementsDatasetReduce):
         applied = (func(ds, *args, **kwargs) for ds in self._iter_grouped())
         return self._combine(applied)
 
+    def apply(self, func, args=(), shortcut=None, **kwargs):
+        """
+        Backward compatible implementation of ``map``
+
+        See Also
+        --------
+        DatasetGroupBy.map
+        """
+
+        warnings.warn(
+            "GroupBy.apply may be deprecated in the future. Using GroupBy.map is encouraged",
+            PendingDeprecationWarning,
+            stacklevel=2,
+        )
+        return self.map(func, shortcut=shortcut, args=args, **kwargs)
+
     def _combine(self, applied):
         """Recombine the applied objects like the original."""
         applied_example, applied = peek_at(applied)
@@ -782,7 +965,7 @@ class DatasetGroupBy(GroupBy, ImplementsDatasetReduce):
             Function which can be called in the form
             `func(x, axis=axis, **kwargs)` to return the result of collapsing
             an np.ndarray over an integer valued axis.
-        dim : str or sequence of str, optional
+        dim : `...`, str or sequence of str, optional
             Dimension(s) over which to apply `func`.
         axis : int or sequence of int, optional
             Axis(es) over which to apply `func`. Only one of the 'dimension'
@@ -801,17 +984,7 @@ class DatasetGroupBy(GroupBy, ImplementsDatasetReduce):
             Array with summarized data and the indicated dimension(s)
             removed.
         """
-        if dim == DEFAULT_DIMS:
-            dim = ALL_DIMS
-            # TODO change this to dim = self._group_dim after
-            # the deprecation process. Do not forget to remove _reduce_method
-            warnings.warn(
-                "Default reduction dimension will be changed to the "
-                "grouped dimension in a future version of xarray. To "
-                "silence this warning, pass dim=xarray.ALL_DIMS "
-                "explicitly.",
-                FutureWarning, stacklevel=2)
-        elif dim is None:
+        if dim is None:
             dim = self._group_dim
 
         if keep_attrs is None:
@@ -819,25 +992,10 @@ class DatasetGroupBy(GroupBy, ImplementsDatasetReduce):
 
         def reduce_dataset(ds):
             return ds.reduce(func, dim, keep_attrs, **kwargs)
-        return self.apply(reduce_dataset)
 
-    # TODO remove the following class method and DEFAULT_DIMS after the
-    # deprecation cycle
-    @classmethod
-    def _reduce_method(cls, func, include_skipna, numeric_only):
-        if include_skipna:
-            def wrapped_func(self, dim=DEFAULT_DIMS,
-                             skipna=None, **kwargs):
-                return self.reduce(func, dim,
-                                   skipna=skipna, numeric_only=numeric_only,
-                                   allow_lazy=True, **kwargs)
-        else:
-            def wrapped_func(self, dim=DEFAULT_DIMS,  # type: ignore
-                             **kwargs):
-                return self.reduce(func, dim,
-                                   numeric_only=numeric_only, allow_lazy=True,
-                                   **kwargs)
-        return wrapped_func
+        check_reduce_dims(dim, self.dims)
+
+        return self.map(reduce_dataset)
 
     def assign(self, **kwargs):
         """Assign data variables by group.
@@ -846,7 +1004,7 @@ class DatasetGroupBy(GroupBy, ImplementsDatasetReduce):
         --------
         Dataset.assign
         """
-        return self.apply(lambda ds: ds.assign(**kwargs))
+        return self.map(lambda ds: ds.assign(**kwargs))
 
 
 ops.inject_reduce_methods(DatasetGroupBy)
