@@ -15,18 +15,16 @@ class DummyArray:
         if self.failures:
             self.failures -= 1
             raise DummyFailure
-        return 'success'
+        return "success"
 
 
 def test_robust_getitem():
     array = DummyArray(failures=2)
     with pytest.raises(DummyFailure):
         array[...]
-    result = robust_getitem(
-        array, ..., catch=DummyFailure, initial_delay=1)
-    assert result == 'success'
+    result = robust_getitem(array, ..., catch=DummyFailure, initial_delay=1)
+    assert result == "success"
 
     array = DummyArray(failures=3)
     with pytest.raises(DummyFailure):
-        robust_getitem(array, ..., catch=DummyFailure, initial_delay=1,
-                       max_retries=2)
+        robust_getitem(array, ..., catch=DummyFailure, initial_delay=1, max_retries=2)
